﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    /// <summary>
    /// Pay对象是 Payment与Account之间互相转变的动作
    /// 或者Account与Account之间转账
    /// 此动作可附带Biz相关内容,比如Settlement
    /// </summary>
    [Api(Description = "支付")]
    [Alias(Configs.TableNamePay)]
    public class Pay : EntityWithMetadata, IEntityInAssemblyDb, IEntityHasBiz,
        IEntityHasOffset, IEntityHasSummary,
        IEntityHasRemark, IEntityHasCreator, IEntityHasUpdater
    {
        #region 关联信息 Owner,Biz
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [InternalSetter]
        [StringLength(500)]
        public string OwnerName { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OffsetKvid { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [InternalSetter]
        [StringLength(100)]
        public string BizType { get; set; }
        #endregion

        #region 收付双方
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayerKvid { get; set; }

        [ApiMember(Description = "付方单位")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayerName { get; set; }

        [ApiMember(Description = "付方来源")]
        [StringLength(50), Required]
        public PayType FromType { get; set; }

        [ApiMember(Description = "付方来源Kvid")]
        [IgnoreUpdate]
        [Required]
        public Guid FromKvid { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayeeKvid { get; set; }

        [ApiMember(Description = "收方单位")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeName { get; set; }

        [ApiMember(Description = "收方来源")]
        [StringLength(50), Required]
        public PayType ToType { get; set; }

        [ApiMember(Description = "收方来源Kvid")]
        [IgnoreUpdate]
        [Required]
        public Guid ToKvid { get; set; }
        #endregion

        #region 基本信息
        [ApiMember(Description = "流水号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string SerialNumber { get; set; }

        [ApiMember(Description = "支付类型")]
        [StringLength(50), Default("")]
        public string Type { get; set; }

        [ApiMember(Description = "支付日期")]
        [InternalSetter]
        public DateTime? PayedTime { get; set; }
        #endregion

        #region 费用相关
        //[ApiMember(Description = "结算金额,结算明细统计")]
        //[InternalSetter]
        //[DecimalLength(15, 2), Default(0)]
        //public decimal AmountSettlement { get; set; }

        [ApiMember(Description = "发生金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "折扣金额,发生金额-实付金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountDiscount { get; set; }

        [ApiMember(Description = "实付金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPayment { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }
        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        //[IgnoreUpdate]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Organization Creator Updater Operator OperateTime 

        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经办人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }

        [ApiMember(Description = "组织机构名称,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [StringLength(200), CurrentOrganizationName]
        public string OrganizationName { get; set; }

        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion

    }

    public enum PayType
    {
        Unsupported = 0,

        /// <summary>
        /// 付款记录支付
        /// </summary>
        Payment,

        /// <summary>
        /// 客户账户支付
        /// </summary>
        Account,

        /// <summary>
        /// 抵扣的
        /// </summary>
        Discount
    }

    public class PayingMethod
    {
        public PayType Type { get; set; }

        public Guid Kvid { get; set; }

        public decimal Amount { get; set; }

        public string Remark { get; set; }
    }
}
